/*******************************************************
 *
 * $Id: FlyingChickenScreen.cpp,v 1.12 2003/05/29 15:41:13 teacy Exp $
 *
 * This file has been placed in the public domain.
 *
 *******************************************************/


#include "./HostControl.h"
#include <PalmOS.h>

#include "FlyingChickenScreen.h"
#include "gfx/SpriteEngine.h"
#include "gfx/Color.h"
#include "sound/SoundEngine.h"
#include "input/ControllerFactory.h"
#include "input/DigitalController.h"


FlyingChickenScreen::FlyingChickenScreen(const StateDescriptor *screenState) : Screen()
{
	state = (FlyingChickenScreenState *)(screenState->state);
	exitScreen = false;
	controller = ControllerFactory::getDigitalController();
}


void FlyingChickenScreen::init(const Canvas *displayCanvas, const Canvas *viewportCanvas)
{
	// Calculate viewport height
	viewportHeight = viewportCanvas->height; 
	viewportWidth = viewportCanvas->width;

	// CAUTION!!! All bitmaps need to be prepared for the Canvas on which they will
	// later be painted. No mixing is allowed!

	// Prepare three animated Sprites (all with the same frames of animation),
	chickenFrames = new AnimFrames(viewportCanvas, 8, 2000, 2010, 20, 20);
	chickenGroup = new SpriteGroup(3);
	chickenGroup->createSprites(chickenFrames, true, true);

	// Place chicken according to state
	chickenGroup->move(CHICKEN1_SPRITE, state->xCoord, state->yCoord);
	chickenGroup->move(CHICKEN2_SPRITE, 80 - (state->xCoord - 80), state->yCoord + 20);
	chickenGroup->move(CHICKEN3_SPRITE, state->xCoord, state->yCoord);

	// Prepare Tux for the background
	tuxBitmap = viewportCanvas->prepareSolidBitmap(3000);	

	// Prepare coin bitmap for the fixed overlay
	coinBitmap = displayCanvas->prepareTransparentBitmap(3100, 3101);

	SoundEngine::playSong(0);
}


FlyingChickenScreen::~FlyingChickenScreen()
{
	delete tuxBitmap;
	delete coinBitmap;
	delete chickenGroup;
	delete chickenFrames;

	delete controller;

	SoundEngine::stopSong();
}



	
Boolean FlyingChickenScreen::nextPeriod()
{
	state->periodCounter++;

	const int direction = controller->getDirection();

	if (direction != DigitalController::DIR_RELEASED)
	{
		state->lastMoved = state->periodCounter;
				
		if (direction & DigitalController::DIR_LEFT)
			state->xCoord -= 4;
		else if (direction & DigitalController::DIR_RIGHT)
			state->xCoord += 4;

		if (direction & DigitalController::DIR_DOWN)
			state->yCoord += 4;
		else if (direction & DigitalController::DIR_UP)
			state->yCoord -= 4;
	}


	if (controller->isButtonPressed(0))
	{
		SoundEngine::playFx(0);
		chickenGroup->setVisibility(CHICKEN1_SPRITE, !(chickenGroup->isVisible(CHICKEN1_SPRITE)));
	}


	if (controller->isButtonPressed(1))
	{
		exitScreen = true;		
	}

				
	// Walk around if the user hasn't moved us for a while...
	if ((state->periodCounter - state->lastMoved) > 100)
	{
		state->xCoord -= 4;  
		state->yCoord += 4;
	}


	// Wrap around at screen edge
	if (state->xCoord > (viewportWidth + 40))
		state->xCoord = -40;
	if (state->xCoord < -40)
		state->xCoord = (viewportWidth + 40);

	if (state->yCoord > (viewportHeight + 60))
		state->yCoord = -60;
	if (state->yCoord < -60)
		state->yCoord = (viewportHeight + 60);
		
	chickenGroup->move(CHICKEN1_SPRITE, state->xCoord, state->yCoord);
	chickenGroup->move(CHICKEN2_SPRITE, state->xCoord + 21, (viewportHeight >> 1) - (state->yCoord - (viewportHeight >> 1)));
	chickenGroup->move(CHICKEN3_SPRITE, state->xCoord + 40, (viewportHeight >> 1));

	UInt16 frameNumber = (state->periodCounter >> 2) % 14;
	if (frameNumber > 7)
		frameNumber = 14 - frameNumber;
	chickenGroup->setFrame(CHICKEN1_SPRITE, frameNumber);
	chickenGroup->setFrame(CHICKEN3_SPRITE, frameNumber);

	frameNumber = ((state->periodCounter + 10) >> 2) % 14;
	if (frameNumber > 7)
		frameNumber = 14 - frameNumber;
	chickenGroup->setFrame(CHICKEN2_SPRITE, frameNumber);


	return exitScreen;
}


void FlyingChickenScreen::getViewportProperties(const Canvas *displayCanvas, ViewportProperties& viewport) const
{
	viewport.region.topLeft.x =  0;
	viewport.region.topLeft.y = 24;
	viewport.region.extent.x = displayCanvas->width;
	viewport.region.extent.y = displayCanvas->height - 48;
}


void FlyingChickenScreen::drawBackground(const Canvas *viewportCanvas) const
{
	viewportCanvas->fillRectangle(Color::WHITE, 0, 0, viewportCanvas->width - 1, viewportCanvas->height - 1);
	viewportCanvas->drawBitmap(tuxBitmap, -8, 10);
	viewportCanvas->drawBitmap(tuxBitmap, 111, 20); 
	viewportCanvas->drawBitmap(tuxBitmap, 16, 30);
}


void FlyingChickenScreen::drawAction(const Canvas * /* viewportCanvas */, RectangleType *bounds)
{
	// Restore background
	chickenGroup->restoreBackground();

	// Paint sprites at new position
	chickenGroup->draw(bounds);
}


void FlyingChickenScreen::drawFixedOverlays(const Canvas *displayCanvas) const
{
	displayCanvas->fillRectangle(Color::BLACK, 0, 0, displayCanvas->width - 1, 23);
	displayCanvas->drawText(Color::WHITE, Color::BLACK, stdFont, "Score", 5, 1, 1);
	displayCanvas->fillRectangle(Color::BLACK, 0, displayCanvas->height - 24, displayCanvas->width - 1, displayCanvas->height - 1);

	// Draw a coin
	displayCanvas->drawBitmap(0, coinBitmap, 100, 2);
}


void FlyingChickenScreen::drawDynamicOverlays(const Canvas *displayCanvas) const
{
	// Draw a fake "score"
	Char frameChar[10];
	StrPrintF(frameChar, "%6lu", state->periodCounter);
	displayCanvas->drawText(Color::WHITE, Color::BLACK, stdFont, frameChar, 6, 1, 10);
}


void FlyingChickenScreen::receiveEvent(const EventType* evtPtr)
{
	// Controller needs to be able to see the event
	controller->receiveEvent(evtPtr);
}


StateDescriptor *FlyingChickenScreen::createStateDescriptor() 
{	
	createState(FlyingChickenScreenState);

	// Initialize state
	state->periodCounter = state->lastMoved = 0;
	state->xCoord = 80;
	state->yCoord = 80; 

	return stateDescriptor;
}
