/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: SpriteEngine.cpp,v 1.3 2003/03/17 21:15:38 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/

#include "Customization.h"

#include "gfx/SpriteEngine.h"
#include "gfx/Canvas.h"
#include "gfx/Geometry.h"
#include "sound/SoundEngine.h"        // Needed for invocations of SoundEngine::timeTick()
#include "Utilities.h"
#include "Device.h"


// ****************************************************************************************************
//
//       Class AnimFrame
//
// ****************************************************************************************************

AnimFrames::AnimFrames(const Canvas *canvas, UInt16 numFrames, DmResID bitmapID, DmResID maskID, Coord hotSpotX, Coord hotSpotY)
{
	init(canvas, numFrames, bitmapID, maskID, hotSpotX, hotSpotY, drawMASK);
}


AnimFrames::AnimFrames(const Canvas *canvas, UInt16 numFrames, DmResID bitmapID, Coord hotSpotX, Coord hotSpotY)
{
	init(canvas, numFrames, bitmapID, -1, hotSpotX, hotSpotY, drawNO_MASK);
}


void AnimFrames::init(const Canvas *canvas, UInt16 numFrames, DmResID bitmapID, DmResID maskID, Coord hotSpotX, Coord hotSpotY, DrawingMode drawingMode)
{
	this->canvas = canvas;

	this->hotSpotX  = hotSpotX;
	this->hotSpotY  = hotSpotY;		
	this->drawingMode = drawingMode;

	///////////////////////////////
	// Prepare bitmaps for blitter
	///////////////////////////////
	switch(drawingMode)
	{
		case AnimFrames::drawMASK:
			bitmapArray = canvas->prepareTransparentBitmaps(numFrames, bitmapID, maskID);
		break;
		case AnimFrames::drawNO_MASK:
			bitmapArray = canvas->prepareSolidBitmaps(numFrames, bitmapID);
		break;
	}
}


AnimFrames::~AnimFrames()
{
	delete bitmapArray;
}


Boolean AnimFrames::getBounds(Coord x, Coord y, RectangleType *bounds) const
{
	// Only return bounds if the painted frame is visible.
	Int16 topLeftX = x - hotSpotX;
	
	if (topLeftX < canvas->width)
	{
		Int16 topLeftY = y - hotSpotY;

		Int16 width = bitmapArray->width;
		Int16 height = bitmapArray->height;
		
		if (topLeftY < canvas->height)
		{
			Int16 bottomRightX = topLeftX + width;
			
			if (bottomRightX > 0)
			{
				Int16 bottomRightY = topLeftY + height;
		
				if (bottomRightY > 0)
				{
					if (bounds != NULL)
					{
						bounds->topLeft.x = topLeftX;
						bounds->topLeft.y = topLeftY;
						bounds->extent.x = width;
						bounds->extent.y = height;
					}

					return false;
				}
			}
		}
	}
	
	if (bounds != NULL)
	{
		bounds->topLeft.x = 0;
		bounds->topLeft.y = 0;
		bounds->extent.x  = 0;
		bounds->extent.y  = 0;
	}
	
	return true;
}


void AnimFrames::draw(UInt16 index, Coord x, Coord y, RectangleType *bounds) const
{
	if(getBounds(x, y, bounds))
	{
		return;
	}

	canvas->drawBitmap(index, bitmapArray, x - hotSpotX, y - hotSpotY);
	SoundEngine::timeTick();	
}


// ****************************************************************************************************
//
//       Class Sprite
//
// ****************************************************************************************************


// ----------------------------- Lifecycle management -------------------------------------------------------

Sprite::Sprite()
{
}
	

Sprite::Sprite(const AnimFrames *animFrames, Boolean visible, Boolean saveBackground)
{
	init(animFrames, visible, saveBackground);
}


void Sprite::init(const AnimFrames *animFrames, Boolean visible, Boolean saveBackground)
{
	this->animFrames = animFrames;
	
	this->visible = visible;

	if (saveBackground)
	{
		this->bgSaveState = Sprite::bgNOT_SAVED;
		this->bgSaveBuffer = animFrames->canvas->prepareSaveBuffers(1, animFrames->bitmapArray->width, animFrames->bitmapArray->height);
	}
	else
	{
		this->bgSaveState = Sprite::bgDONT_SAVE;
	}
	
	move(0,0);
	setFrame(0);
}



Sprite::~Sprite()
{
	if(bgSaveState != Sprite::bgDONT_SAVE)
	{
		delete bgSaveBuffer;
	}
}


// ----------------------------- State management (visibility, position, etc.) ------------------------------

void Sprite::show()
{
	visible = true;
}


void Sprite::hide()
{
	visible = false;
}


Boolean Sprite::setVisibility(Boolean visible)
{
	Boolean oldVisible = visible;
	this->visible = visible;
		
	return oldVisible;
}


Boolean Sprite::isVisible() const
{
	return visible;
}


void Sprite::setFrame(UInt16 frameIndex)
{
	this->frameIndex = frameIndex;
}


void Sprite::move(Coord x, Coord y)
{
	this->x = x;
	this->y = y;
}


// ----------------------------- Screen output --------------------------------------------------------------

void Sprite::draw(RectangleType *bounds)
{
	if (!visible)
	{
		if (bounds != NULL)
		{
			bounds->topLeft.x = 0;
			bounds->topLeft.y = 0;
			bounds->extent.x = 0;
			bounds->extent.y = 0;
		}


		if (bgSaveState != Sprite::bgDONT_SAVE)
		{
			bgSaveState = Sprite::bgNOT_SAVED;
		}
			
		return;
	}

	saveBackground();
	animFrames->draw(frameIndex, x, y, bounds);
}


// ----------------------------- Background save / restore --------------------------------------------------

void Sprite::saveBackground()
{
	if (bgSaveState == Sprite::bgDONT_SAVE)
	{
		return;
	}

	Coord bgSaveX = x - animFrames->hotSpotX;
	Coord bgSaveY = y - animFrames->hotSpotY;
	animFrames->canvas->saveBitmap(0, bgSaveBuffer, bgSaveX, bgSaveY);

	bgSaveState = Sprite::bgSAVED;	
}



void Sprite::restoreBackground() const
{
	if (bgSaveState != Sprite::bgSAVED)
	{
		return;
	}

	animFrames->canvas->restoreBitmap(0, bgSaveBuffer);
}


void Sprite::discardBackground()
{
	if (bgSaveState == Sprite::bgSAVED)
	{
		bgSaveState = Sprite::bgNOT_SAVED;
	}
}


Boolean Sprite::getBounds(RectangleType *bounds) const
{
	return animFrames->getBounds(x, y, bounds);
}



// ****************************************************************************************************
//
//       Class SpriteGroup
//
// ****************************************************************************************************

SpriteGroup::SpriteGroup(UInt16 numSprites)
{
	ErrNonFatalDisplayIf(numSprites == 0, "numSprites == 0");	
	this->sprites = new Sprite[numSprites];
	this->numSprites = numSprites;
}
	
	
SpriteGroup::~SpriteGroup()
{
	delete [] sprites;
}

void SpriteGroup::createSprites(const AnimFrames *animFrames, Boolean visible, Boolean saveBackground)
{
	for (UInt16 i = 0; i < numSprites; i++)
	{
		this->sprites[i].init(animFrames, visible, saveBackground);
	}
}

void SpriteGroup::createSprite(UInt16 spriteIdx, const AnimFrames *animFrames, Boolean visible, Boolean saveBackground)
{
	this->sprites[spriteIdx].init(animFrames, visible, saveBackground);
}


void SpriteGroup::show(UInt16 spriteIdx)
{
	this->sprites[spriteIdx].show();
}


void SpriteGroup::hide(UInt16 spriteIdx)
{
	this->sprites[spriteIdx].hide();
}


Boolean SpriteGroup::setVisibility(UInt16 spriteIdx, Boolean visible)
{
	return this->sprites[spriteIdx].setVisibility(visible);
}


Boolean SpriteGroup::isVisible(UInt16 spriteIdx) const
{
	return this->sprites[spriteIdx].isVisible();
}


void SpriteGroup::setFrame(UInt16 spriteIdx, UInt16 frameIndex)
{
	this->sprites[spriteIdx].setFrame(frameIndex);
}


void SpriteGroup::move(UInt16 spriteIdx, Coord x, Coord y)
{
	this->sprites[spriteIdx].move(x, y);
}



void SpriteGroup::draw(RectangleType *bounds)
{
	if (bounds != NULL)
	{
		RectangleType spriteBounds;
		for(UInt16 i = 0; i < numSprites; i++)
		{
			this->sprites[i].draw(&spriteBounds);
			Geometry::uniteBounds(&spriteBounds, bounds, bounds);
		}
	}
	else
	{
		for(UInt16 i = 0; i < numSprites; i++)
		{
			this->sprites[i].draw();
		}	
	} 
}


void SpriteGroup::restoreBackground() const
{
	// Counting towards zero requires a signed(!) count index!!!
	for(Int16 i = numSprites - 1; i >= 0; i--)
	{
		this->sprites[i].restoreBackground();
	}	
}
