/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: PalmOSBlitter.cpp,v 1.4 2003/05/29 15:45:25 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/

/*
 * SEE:
 * - http://www.palmos.com/dev/support/docs/recipes/copying_bitmap.html
 * - http://www.palmos.com/dev/support/docs/recipes/using_winpaintbitmap.html
 */

#include "./HostControl.h"
#include "Customization.h"
#include <BmpGlue.h>

#include "gfx/PalmOSBlitter.h"
#include "gfx/BlitterSupport.h"
#include "gfx/Display.h"
#include "gfx/Canvas.h"
#include "Device.h"

#ifndef USE_SDK50
#define nativeFormat screenFormat
#endif

using namespace prvrazor;



PalmOSTransparentBitmapArray::PalmOSTransparentBitmapArray(const Canvas *canvas, UInt16 numBitmaps, UInt16 bitmapID, UInt16 maskID)
	: BitmapArray(numBitmaps), bitmaps(new PalmOSTransparentBitmap[numBitmaps])
{
	{
		const MemHandle resH = DmGet1Resource( bitmapRsc, bitmapID);
		ErrNonFatalDisplayIf( !resH, "Missing bitmap" );
		BitmapType *const resP = static_cast<BitmapType *>(MemHandleLock(resH));
		
		BmpGlueGetDimensions(resP, &width, &height, NULL);
		MemPtrUnlock(resP);
		DmReleaseResource( resH );
	}
	
	RectangleType bitmapRect;
	bitmapRect.topLeft.x = bitmapRect.topLeft.y = 0;
	bitmapRect.extent.x = width; bitmapRect.extent.y = height;

	for (UInt16 i = 0; i < numBitmaps; i++)
	{
		PalmOSTransparentBitmap *const bitmap = const_cast<PalmOSTransparentBitmap *const>(&bitmaps[i]);

		// Paint sprite-mask to offscreen window
		{		
			const MemHandle resH = DmGet1Resource(bitmapRsc, maskID + i);
			ErrNonFatalDisplayIf( !resH, "Missing mask" );
			BitmapType *const resP = static_cast<BitmapType *>(MemHandleLock(resH));
	
			UInt16 error = errNone;
			bitmap->mask = WinCreateOffscreenWindow(width, height, nativeFormat, &error);
			if (error != errNone)
			{
				Device::panic("Not enough memory for mask.");
			}				
	
			WinSetDrawWindow(bitmap->mask);
	
			if (Device::supports35)
			{
				WinSetForeColor((1 << Display::depth) - 1);
				WinSetBackColor(0);
			}
				
			WinEraseRectangle(&bitmapRect, 0);
			WinDrawBitmap (resP, 0,0);
	
			MemPtrUnlock(resP);
			DmReleaseResource( resH );				
		}

		// Paint sprite-bitmap to offscreen window
		{
			const MemHandle resH = DmGet1Resource( bitmapRsc, bitmapID + i);
			ErrNonFatalDisplayIf( !resH, "Missing bitmap" );
			BitmapType *const resP = static_cast<BitmapType *>(MemHandleLock(resH));

			UInt16 error = errNone;
			bitmap->bitmap = WinCreateOffscreenWindow(width, height, nativeFormat, &error);
			if (error != errNone)
			{
				Device::panic("Not enough memory for bitmap.");
			}				
	
			WinSetDrawWindow(bitmap->bitmap);
			if (Device::supports35 && (Display::depth == 1))
			{
				// Required for 1bpp hires test case in OS5 compatibility test kit.
				WinSetForeColor(1);
				WinSetBackColor(0);		
			}
	
			WinDrawBitmap (resP, 0,0);
	
			// TODO: Figure out why this has no effect on OS 3.0 device at 2bpp.
			WinCopyRectangle(bitmap->mask, bitmap->bitmap, &bitmapRect, 0, 0, winErase);	
	
			MemPtrUnlock(resP);
			DmReleaseResource( resH );	
		}
	}

	WinSetDrawWindow(canvas->drawWindow);		
}



PalmOSTransparentBitmapArray::~PalmOSTransparentBitmapArray()
{
	for (UInt16 i = 0; i < numBitmaps; i++)
	{
		const PalmOSTransparentBitmap *const bitmap = &bitmaps[i];
		
		WinDeleteWindow(bitmap->bitmap, false);			
		WinDeleteWindow(bitmap->mask, false);	
	}

	delete [] bitmaps;
}



void PalmOSTransparentBitmapArray::draw(const Canvas *canvas, UInt16 index, Coord x, Coord y) const
{
	const PalmOSTransparentBitmap *const bitmap = &bitmaps[index];

	RectangleType size;
	size.topLeft.x = 0;
	size.topLeft.y = 0;
	size.extent.x = width;
	size.extent.y = height;			

	if (Device::supports35 && (Display::depth == 1))
	{
		WinSetForeColor(1);
		WinSetBackColor(0);		
	}

	const WinHandle drawWindow = canvas->drawWindow;
	WinCopyRectangle(bitmap->mask, drawWindow, &size, x, y, winMask);
	WinCopyRectangle(bitmap->bitmap, drawWindow, &size, x, y, winOverlay);		
}



PalmOSSolidBitmapArray::PalmOSSolidBitmapArray(const Canvas */*canvas*/, UInt16 numBitmaps, UInt16 bitmapID)
	: BitmapArray(numBitmaps), bitmaps(new PalmOSSolidBitmap[numBitmaps])
{
	for (UInt16 i = 0; i < numBitmaps; i++)
	{
		PalmOSSolidBitmap *const bitmap = const_cast<PalmOSSolidBitmap *const>(&bitmaps[i]);
		
		bitmap->bitmapResH = DmGet1Resource( bitmapRsc, bitmapID + i);
		bitmap->bitmap = static_cast<BitmapType *>(MemHandleLock(bitmap->bitmapResH));
		ErrNonFatalDisplayIf( !(bitmap->bitmap), "Missing bitmap" );		

#if ERROR_CHECK_LEVEL == ERROR_CHECK_FULL
		// Point out problem with OS 3.0. Compressed bitmaps cannot be drawn to negative coordinates
		if ((BlitterSupport::getPixelSize(bitmap->bitmap) < 4) && BlitterSupport::isCompressed(bitmap->bitmap))
		{
			ErrNonFatalDisplay("Problematic bitmap: 1 or 2bpp, lo-res, compressed");
		}
#endif
	}

	BmpGlueGetDimensions(bitmaps->bitmap, &width, &height, NULL);
}




PalmOSSolidBitmapArray::~PalmOSSolidBitmapArray()
{
	for (UInt16 i = 0; i < numBitmaps; i++)
	{
		const PalmOSSolidBitmap *const bitmap = &bitmaps[i];
		
		MemPtrUnlock(bitmap->bitmap);
		DmReleaseResource(bitmap->bitmapResH);		
	}

	delete [] bitmaps;
}


void PalmOSSolidBitmapArray::draw(const Canvas */*canvas*/, UInt16 index, Coord x, Coord y) const
{
	const PalmOSSolidBitmap *const bitmap = &bitmaps[index];

	// Needed for PalmOS 5 Readiness Test Kit
	if (Device::supports35 && (Display::depth == 1))
	{
		WinSetForeColor(1);  // Black
		WinSetBackColor(0);	 // White
	}

	WinDrawBitmap(bitmap->bitmap, x, y);
}





PalmOSSaveBufferArray::PalmOSSaveBufferArray(UInt16 numBuffers, Coord width, Coord height)
    : SaveBufferArray(numBuffers, width, height), buffers(new PalmOSSaveBuffer[numBuffers])
{
	for (UInt16 i = 0; i < numBuffers; i++)
	{
		PalmOSSaveBuffer *const buffer = const_cast<PalmOSSaveBuffer *>(&buffers[i]);
		UInt16 error;
		buffer->buffer = WinCreateOffscreenWindow(width, height, nativeFormat, &error);
		if (error != errNone)
		{
			Device::panic("Not enough memory for save buffer.");
		}		
	}
}


PalmOSSaveBufferArray::~PalmOSSaveBufferArray()
{
	for (UInt16 i = 0; i < numBuffers; i++)
	{
		PalmOSSaveBuffer *buffer = &buffers[i];		
		WinDeleteWindow(buffer->buffer, false);			
	}
	
	delete [] buffers;
}


void PalmOSSaveBufferArray::save(const Canvas *canvas, UInt16 index, Coord x, Coord y) const
{
	RectangleType saveBounds;
	saveBounds.topLeft.x = x;
	saveBounds.topLeft.y = y;
	saveBounds.extent.x = width;
	saveBounds.extent.y = height;

	PalmOSSaveBuffer *const buffer = &buffers[index];		
	buffer->x = x;
	buffer->y = y;

	WinCopyRectangle(canvas->drawWindow, buffer->buffer, &saveBounds, 0, 0, winPaint);
}


void PalmOSSaveBufferArray::restore(const Canvas *canvas, UInt16 index) const
{
	RectangleType srcRect;
	srcRect.topLeft.x = 0;
	srcRect.topLeft.y = 0;
	srcRect.extent.x = width;
	srcRect.extent.y = height;
	
	PalmOSSaveBuffer *const buffer = &buffers[index];		

	WinCopyRectangle(buffer->buffer, canvas->drawWindow, &srcRect, buffer->x, buffer->y, winPaint);
}





PalmOSBlitter::PalmOSBlitter(Canvas *canvas) : Blitter(canvas)
{
}


PalmOSBlitter::~PalmOSBlitter()
{
}



const BitmapArray *PalmOSBlitter::prepareTransparentBitmaps(UInt16 numBitmaps, DmResID bitmapID, DmResID maskID) const
{
	return new PalmOSTransparentBitmapArray(canvas, numBitmaps, bitmapID, maskID);
}


const BitmapArray *PalmOSBlitter::prepareSolidBitmaps(UInt16 numBitmaps, DmResID bitmapID) const
{
	return new PalmOSSolidBitmapArray(canvas, numBitmaps, bitmapID);
}


void PalmOSBlitter::drawText(Color textColor, Color bgColor, FontID font, const Char *chars, Int16 len, Coord x, Coord y) const
{
	FntSetFont(font);	

	if (Device::supports35)
	{
		canvas->setBackgroundColor(bgColor);
		canvas->setForegroundColor(textColor);
		WinDrawChars(chars, len, x, y);
	}
	else
	{
		if (canvas->setForegroundColor(textColor))
		{
			WinDrawChars(chars, len, x, y);
		}
		else
		{
			WinDrawInvertedChars(chars, len, x, y);
		}
	}	
}	


void PalmOSBlitter::fillRectangle(Color color, Coord topX, Coord topY, Coord bottomX, Coord bottomY) const
{
	RectangleType rectangle;
	rectangle.topLeft.x = topX;
	rectangle.topLeft.y = topY;
	rectangle.extent.x = (bottomX - topX) + 1;
	rectangle.extent.y = (bottomY - topY) + 1;
	
	if (Device::supports35)
	{
		canvas->setForegroundColor(color);
		WinDrawRectangle(&rectangle, 0);
	}
	else
	{
		if (canvas->setForegroundColor(color))
		{
			WinDrawRectangle(&rectangle, 0);
		}
		else
		{
			WinEraseRectangle(&rectangle, 0);
		}
	}
}


SaveBufferArray *PalmOSBlitter::prepareSaveBuffers(UInt16 numBuffers, Coord width, Coord height) const
{
	return new PalmOSSaveBufferArray(numBuffers, width, height);
}
