/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: ActionEngine.h,v 1.3 2003/03/17 22:16:24 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


#ifndef ACTIONENGINE_H
#define ACTIONENGINE_H

#include "Sections.h"
#include <PalmOS.h>

#include "StateDescriptor.h"
#include "Screen.h"


/**
 * The base class for a game.
 * <br>
 * Defines the sequence of screens, and takes care of persistence.
 */
class ActionEngine
{
	public:
/// @name Lifecycle management
//@{
	/**
	 * Construct a new ActionEngine
	 */
	ActionEngine(UInt16 numStateDescriptors) SEC_GAME; 


	/**
	 * Destroy the ActionEngine
	 */
	virtual ~ActionEngine() SEC_GAME = 0;
//@}

	
/// @name Control flow
//@{
	/**
	 * Initialize the ActionEngine and all Screens.
	 * Make sure you set the persistent state of the ActionEngine and
	 * all Screens to working default values.
	 */
	virtual void init() SEC_GAME = 0;

	/**
	 * Get the currently needed Screen
	 * Ownership of the screen is transferred to the caller.
	 */
	 virtual Screen* getCurrentScreen() const SEC_GAME = 0;

	/**
	 * Get the next Screen in the sequence of Screens.
	 * Ownership of the screen is transferred to the caller.
	 */
  	 virtual Screen *getNextScreen() SEC_GAME = 0;
//@}


/// @name Persistence
//@{
	/**
	 * The version number of the application state. 
	 *
	 * This should be increased by 1 every time the application's 
	 * state descriptor layout changes. 
	 *
	 * @return a version number 
	 */ 
	virtual UInt16 getStateDescriptorsVersion() const SEC_GAME = 0;
	

	/**
	 * Return the nth state descriptor for this ActionEngine.
	 * Ownership of the descriptor remains with the ActionEngine.
	 *
	 * @return a StateDescriptor
	 */	 
	StateDescriptor *getStateDescriptor(UInt16 index) const SEC_GAME;
//@}


/// @name Miscellaneous
//@{
	virtual Boolean getPreventAutoOff() const SEC_GAME;
//@}


	protected:
	StateDescriptorPtr *stateDescriptors;

	private:
	UInt16 numStateDescriptors;
};



#define MYACTIONENGINE_PUBLIC_OPS  \
	MyActionEngine() SEC_GAME;  \
	~MyActionEngine() SEC_GAME; \
	void init() SEC_GAME; \
	Screen *getCurrentScreen() const SEC_GAME; \
	Screen *getNextScreen() SEC_GAME; \
\
	UInt16 getStateDescriptorsVersion() const SEC_GAME;





#endif
