/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: Canvas.h,v 1.5 2003/05/30 12:30:21 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/

/*
 TODO: Is _drawBitmap unneccessary?
 TODO: Should argument order be moved to allow defaults (index, numBitmaps)?
 TODO: How can text with transparent background be realized?
 TODO: Should drawText rather work with a 0x00-terminated string?
 TODO: Color params can be made constant
 */


#ifndef CANVAS_H
#define CANVAS_H

#include <PalmOS.h>
#include "RazorSections.h"

#include "gfx/Geometry.h"
#include "gfx/Color.h"


namespace prvrazor
{
	class Blitter;
}

using namespace prvrazor;


class Canvas;


class BitmapArray
{
	public:
	virtual ~BitmapArray() SEC_RAZOR_INIT = 0;
	virtual void draw(const Canvas *canvas, UInt16 index, Coord x, Coord y) const = 0;

	Coord width;
	Coord height;
	UInt16 numBitmaps;	

    protected:
	BitmapArray(UInt16 numBitmaps) SEC_RAZOR_INIT;
};



class SaveBufferArray
{
	public:
    virtual ~SaveBufferArray() SEC_RAZOR_INIT = 0;
    virtual void save(const Canvas *canvas, UInt16 index, Coord x, Coord y) const = 0;
    virtual void restore(const Canvas *canvas, UInt16 index) const = 0;

	Int16 width;
	Int16 height;
	UInt16 numBuffers;

    protected:
    SaveBufferArray(UInt16 numBuffers, Coord width, Coord height) SEC_RAZOR_INIT;
};


/**
 * Canvas represents a drawing area.
 * <br>
 * An instance of Canvas has properties, such as width and height, 
 * and a few basic drawing operations.
 * <br>
 * It can save/restore parts of itself.
 * <br>
 * It may provide access to its screen memory, to enable drawing
 * through direct memory manipulations.
 */
class Canvas
{
	public:

    enum CoordSystem {COORDS_STANDARD = 1, COORDS_NATIVE = 2};

///@name Construction / Destruction
//@{
	/**
	 * Construct a new Canvas.
	 */
	Canvas(Coord width, Coord height, CoordSystem coordSystem) SEC_RAZOR_INIT;

	/**
	 * Destroy the Canvas.
	 */
	virtual ~Canvas() SEC_RAZOR_INIT;
//@}

///@name Coordinate systems
//@{
    virtual void setCoordSystem(CoordSystem coordSystem) SEC_RAZOR_INIT;
//@}

///@name Draw operations
//@{
	const BitmapArray *prepareTransparentBitmaps(UInt16 numBitmaps, DmResID bitmapID, DmResID maskID) const SEC_RAZOR_INIT;
	const BitmapArray *prepareTransparentBitmap(DmResID bitmapID, DmResID maskID) const SEC_RAZOR_INIT;
	const BitmapArray *prepareSolidBitmaps(UInt16 numBitmaps, DmResID bitmapID) const SEC_RAZOR_INIT;
	const BitmapArray *prepareSolidBitmap(DmResID bitmapID) const SEC_RAZOR_INIT;

    /**
     * Draw the indexth bitmap from an array of bitmaps.
     * 
     * @param index the index into the array of bitmaps. Must be 0..elements-1
     * @param bitmapArray the array of bitmaps. Must have at least index+1 elements
     * @param x the logical x coordinate
     * @param y the logical y coordinate
     */
	void drawBitmap(UInt16 index, const BitmapArray *bitmapArray, Coord x, Coord y) const SEC_RAZOR;

    /**
     * Draw the first bitmap (index=0) from an array of bitmaps.
     * 
     * @param bitmapArray the array of bitmaps. Must have at least 1 element
     * @param x the logical x coordinate
     * @param y the logical y coordinate
     */
	void drawBitmap(const BitmapArray *bitmapArray, Coord x, Coord y) const SEC_RAZOR;

    /**
     * Draw a text string.
     * 
     * @param textColor the color of the text
     * @param bgColor the color of the background
     * @param font the font (TODO: Which ones are supported on all devices)
     * @param chars the text
     * @param len the length of the text (TODO: Why is this signed int?)
     * @param x the logical x coordinate of the topleft corner of the first character
     * @param y the logical y coordinate of the topleft corner of the first character
     */
	void drawText(Color textColor, Color bgColor, FontID font, const Char *chars, Int16 len, Coord x, Coord y) const SEC_RAZOR;

	void fillRectangle(Color color, Coord topX, Coord topY, Coord bottomX, Coord bottomY) const SEC_RAZOR;

	void drawPolygon(const xy *e, int edgeCount, const Color color) const SEC_RAZOR;
//@}


///@name Save/Restore Area operations
//@{
	SaveBufferArray *prepareSaveBuffers(UInt16 numBuffers, Coord width, Coord height) const SEC_RAZOR_INIT;
	void saveBitmap(UInt16 index, SaveBufferArray *bufferArray, Coord x, Coord y) const SEC_RAZOR;
	void restoreBitmap(UInt16 index, SaveBufferArray *bufferArray) const SEC_RAZOR;
//@}


///@name OS interaction
//@{
    /**
     * Use this Canvas as the default canvas for OS drawing operations.
     */
	virtual void setOSCanvas() const = 0;

	/**
	 * Make the underlying OS use the color as foreground color. Returns a special indicator for 1bpp devices.
	 *
	 * @return true = color is dark, false = color is light.
	 */
	virtual Boolean setForegroundColor(const Color color) const = 0;
	
	/**
	 * Make the underlying OS use the color as background color. Returns a special indicator for 1bpp devices.
	 *
	 * @return true = color is dark, false = color is light.
	 */
	virtual Boolean setBackgroundColor(const Color color) const = 0;
//@}



	Coord width;
	Coord height;
	Coord rows;
	Coord cols;

    Int8 shiftFactor;
    CoordSystem coordSystem;
        

	/**
	 * The handle to the Canvas' window
	 */
	WinHandle drawWindow;

	/**
	 * The pointer to the Canvas' bitmap
	 */
	BitmapType *_drawBitmap;

	/**
	 * The pointer to the Canvas' bitmap bits
	 */
	void *drawBits;

	/**
	 * The number of bytes in a row of the Canvas
	 */	
	UInt16 rowBytes;


	protected:
	/**
	 * Blitter that can be used to draw to the Canvas
	 */
	Blitter *blitter;


    private:
    static int calcShiftFactor(UInt16 coordSystem);
};


#endif
