/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: Display.h,v 1.4 2003/05/30 12:30:21 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/

/*
 TODO: 
    - Should most of Display be made private and be declared as a friend of Presentation?
 */


#ifndef DISPLAY_H
#define DISPLAY_H

#include <PalmOS.h>
#include "RazorSections.h"
#include "gfx/Canvas.h"


/**
 * The properties of the display.
 * Used to communicate required display properties from the Screens to the Display.
 */
struct DisplayProperties
{
    /**
     * The coordinate system.
     * This coordinate system will be used for all coordinate calculations on all canvases.
     */
    Canvas::CoordSystem coordSystem;
    /**
     * Display property flags.
     * This is reserved for future use.
     * TODO: Turn this into a bitfield? Gather ideas for flags.
     */
    UInt16 flags;
};


/**
 * The properties of the viewport.
 * Used to communicate viewport properties from the Screens to the Display.
 */
struct ViewportProperties
{
    /**
     * The size and position of the viewport.
     * All coordinates are in logical units.
     */
	RectangleType region;
};


/**
 * The device's physical display area. It provides access to
 * the physical properties of the area (dimensions, color), and it provides access
 * to the display canvas and the viewport canvas for drawing. It manages the double-buffering
 * functionality for flicker-free drawing.
 * <br>
 * It also enables the framework to setup the physical display. This functionality
 * is not directly accessible.
 */
class Display
{
	public:
    /**
     * Constants to express the display density.
     */
    enum Density {DENSITY_SINGLE = 1, /**< a single density display, used by lores devices */
                  DENSITY_DOUBLE = 2  /**< a double density display, used by hires devices */
                 };


///@name Construction / Destruction
//@{
	/**
	 * Construct a new Display.
	 */
	Display() SEC_RAZOR_INIT;

	/**
	 * Destroy the Display.
	 */
	virtual ~Display() SEC_RAZOR_INIT;
	void destroy() SEC_RAZOR_INIT;
//@}

/// @name Output management
//@{
	/**
	 * Begin drawing into the offscreen viewport
	 */
	void beginDrawViewport();
	
	/**
	 * End drawing into the offscreen viewport
	 * 
	 * @param dirtyRectangle the area that was modified by the viewport draw operations.
	 *               NULL means the entire viewport is dirty.
	 */
	virtual void endDrawViewport(RectangleType *dirtyRectangle);

	/**
	 * Display offscreen viewport
	 */
	virtual void showViewport() = 0;	

    /**
     * Capture the display for use with Razor!
     */
	virtual void captureDisplay() SEC_RAZOR_INIT = 0;

    /**
     * Release the display from use by Razor!
     * Make it usable by the OS again.
     */
	virtual void releaseDisplay() SEC_RAZOR_INIT = 0;
//@}

/// @name Physical Display management
//@{
	virtual void initDisplay(const DisplayProperties& displayProperties) SEC_RAZOR_INIT = 0;
	void initViewport(const ViewportProperties& viewport) SEC_RAZOR_INIT;
//@}

/// @name Canvas management
//@{
    /**
     * Get the display canvas.
     */
	Canvas *getDisplayCanvas() const SEC_RAZOR;

    /**
     * Get the viewport canvas.
     */
	Canvas *getViewportCanvas() const SEC_RAZOR;
//@}

	/**
	 * The physical width (columns) of the display
	 */
	static UInt16 cols;

	/**
	 * The physical height (rows) of the display
	 */
	static UInt16 rows;

	/**
	 * The depth of the display in bpp.
	 */
	static UInt32 depth;

	/**
	 * The density of the display as a Density enumeration member.
	 */
	static Density density;
	
	/**
	 * Is this a color display?
	 */
	static Boolean colorMode;
	
	/**
	 * Is this a direct color (non-palettized) display?
	 */
	static Boolean directColor;

	protected:	
	virtual void createViewportCanvas(const ViewportProperties& viewport) SEC_RAZOR_INIT = 0;
	

	/**
	 * Set the display back to normal
	 */
	virtual void resetDisplay() SEC_RAZOR_INIT = 0;


	// Viewport
	ViewportProperties viewport;	

	// Canvases
	Canvas *displayCanvas;
	Canvas *viewportCanvas;

	// Dirty rectangle management
	// TODO: Should dirty rectangle management be made a task of the Canvas?
	RectangleType currentBounds;
	RectangleType lastBounds;
	RectangleType copyBounds;
};

#endif // DISPLAY_H
