/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: SpriteEngine.h,v 1.2 2003/02/15 12:02:04 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/



#ifndef SPRITE_ENGINE_H
#define SPRITE_ENGINE_H

#include <PalmOS.h>

#include "RazorSections.h"
#include "gfx/Canvas.h"


/**
 * AnimFrames manages the frames of an animated Sprite. Each Sprite is associated
 * with an instance of AnimFrames. Several Sprites may share the same instance of
 * AnimFrames if they have the same appearance. This is recommended, since it saves
 * valuable resources.<br>
 * All frames within an AnimFrames object share the same size, the same hotspot and the same
 * mode of drawing.
 */
class AnimFrames
{
	public:

	/**
	 * Create an instance of a transparent AnimFrames. 
	 * The frames will be constructed from bitmap
	 * resources in a resource database. The bitmap families for the frames need to
	 * be stored at IDs bitmapID..(bitmapID + numFrames) - 1. The 1bpp bitmaps for the
	 * transparency masks need to be stored at IDs maskID..(maskID + numFrames) - 1.
	 *
     * @param numFrames the number of frames
	 * @param bitmapID the ID of the first image bitmap in the resource database.
	 * @param maskID the ID of the first mask bitmap in the resource database
	 * @param hotSpotX the horizontal offset of the hotspot.
	 * @param hotSpotY the vertical offset of the hotspot.
	 */
	AnimFrames(const Canvas *canvas, UInt16 numFrames, DmResID bitmapID, DmResID maskID, Coord hotSpotX = 0, Coord hotSpotY = 0) SEC_RAZOR_INIT;


	/**
	 * Create an instance of an opaque (not transparent) AnimFrames. 
	 * The frames will be constructed from bitmap
	 * resources in a resource database. The bitmap families for the frames need to
	 * be stored at IDs bitmapID..(bitmapID + numFrames) - 1. 
	 *
     * @param numFrames the number of frames
	 * @param bitmapID the ID of the first image bitmap in the resource database.
	 * @param hotSpotX the horizontal offset of the hotspot.
	 * @param hotSpotY the vertical offset of the hotspot.
	 */
	AnimFrames(const Canvas *canvas, UInt16 numFrames, DmResID bitmapID, Coord hotSpotX = 0, Coord hotSpotY = 0) SEC_RAZOR_INIT;


	/**
	 * Destroy the AnimFrames and deallocate all used resources.
	 */
	~AnimFrames() SEC_RAZOR_INIT;


	/**
	 * Get the screen space filled by the AnimFrames when it is placed at the specified coordinates.
	 *
	 * @param x the x coordinate
	 * @param y the y coordinate
     * @param bounds a pointer to a rectangle which will be filled with the screen space
     *        filled by the AnimFrames.
     * @return True, if the frame is completely off the screen, False if it is at least partially on the screen
	 */
	Boolean getBounds(Coord x, Coord y, RectangleType *bounds) const;

	
	/**
	 * Draw a frame at the specified location.
	 * <em>Caution: Although this operation is publically accessible, you should
	 * know exactly what you are doing when you attempt to use it directly!!!</em>
     *
	 * @param index the index of the displayed frame (0..numFrames).
	 * @param x the x coordinate
	 * @param y the y coordinate
     * @param bounds a pointer to a rectangle which will be filled with the screen space
     *        filled by the frame, or NULL.
	 */
	void draw(UInt16 index, Coord x, Coord y, RectangleType *bounds = NULL) const SEC_RAZOR;


	private:

	enum DrawingMode
	{
		drawMASK       = 0,   ///<Draw the frame with a mask for transparency
		drawNO_MASK    = 1    ///<Draw the frames without transparency.
	};


	void init(const Canvas *canvas, UInt16 numFrames, DmResID bitmapID, DmResID maskID, Coord hotSpotX, Coord hotSpotY, DrawingMode drawingMode) SEC_RAZOR_INIT;

	Coord hotSpotX;
	Coord hotSpotY;
	DrawingMode drawingMode;

	const BitmapArray *bitmapArray;

	const Canvas *canvas;

	friend class Sprite;
};



/**
 * Sprite describes a single sprite, i.e. a movable graphical object
 * with a transparent background.
 */
class Sprite
{
	public:

/** @name Lifecycle management
 */
//@{
	/**
	 * Create a single new sprite.
	 * <br>
	 * Look at SpriteGroup if you wish to create a multitude of sprites.
	 *
	 * @param animFrames the animation frames for this sprite
	 * @param visible shall the sprite be visible?
	 * @param saveBackground shall the sprite be able to save and restore its background?
	 */
	Sprite(const AnimFrames *animFrames, Boolean visible, Boolean saveBackground) SEC_RAZOR_INIT;


	/**
	 * Destroy the Sprite and deallocate all used resources.
	 */
	~Sprite() SEC_RAZOR_INIT;
//@}

/** @name State management (visibility, position, etc.)
 */
//@{
	/**
	 * Show the sprite during subsequent draws.
	 */
	void show();


	/**
	 * Hide the sprite during subsequent draws.
	 */
	void hide();


	/**
	 * Set the visibility of the sprite through a flag.
     *
	 * @param visible shall the sprite be visible (true = shown / false = hidden)
	 */
	Boolean setVisibility(Boolean visible);


	/**
	 * Will the sprite be drawn during subsequent draws?
	 */
	Boolean isVisible() const;


	/**
	 * Set the displayed frame of the Sprite.
	 *
	 * @param frameIndex the index of the displayed frame. Must be between 0..numFrames-1 of the associated AnimFrames.
	 */
	void setFrame(UInt16 frameIndex);


	/**
	 * Move the sprite (i.e. its hotspot) to the specified coordinates.
	 */
	void move(Coord x, Coord y);
//@}


/// @name Visual representation
//@{
	/**
	 * Draw the sprite at its current location.
	 * If the sprite has been set to save its background, the background will be
	 * saved before the sprite will be drawn.
     *
     * @param bounds a pointer to a rectangle which will be filled with the screen space
     *        filled by the sprite, or NULL.
	 */
	void draw(RectangleType *bounds = NULL) SEC_RAZOR;


	/**
	 * Restore the saved background of the sprite.
	 * This operation will not do anything if the sprite has been either not drawn, or the
	 * background has been discarded since the last draw operation.
	 */
	void restoreBackground() const SEC_RAZOR;


    /**
     * Throw away the background of a sprite.
     */
	void discardBackground() SEC_RAZOR;


	/**
	 * Get the screen space filled by the sprite.
	 *
     * @param bounds a pointer to a rectangle which will be filled with the screen space
     *        filled by the sprite.
     * @return True, if the Sprite is completely off the screen, False if it is at least partially on the screen
	 */
	Boolean getBounds(RectangleType *bounds) const;
//@}



	private:
	/**
	 * Create a new, disabled sprite
	 */
	Sprite() SEC_RAZOR_INIT;

	/**
	 * Initialize an existing instance with new parameters.
	 */ 
	void init(const AnimFrames *animFrames, Boolean visible = true, Boolean saveBackground = false) SEC_RAZOR_INIT;

	/**
	 * Save the background of the sprite.
	 */
	void saveBackground() SEC_RAZOR;
	

	const AnimFrames *animFrames;

	UInt16 frameIndex;
	Boolean visible;
	Coord x;
	Coord y;


	enum BgSaveState
	{
		bgDONT_SAVE       = 0,   ///<Do not save the background during draw operation
		bgNOT_SAVED       = 1,   ///<The background shall be saved during draw operation, but this has not been done yet
		bgSAVED           = 2    ///<A saved background is available
	};


	BgSaveState bgSaveState;
	SaveBufferArray *bgSaveBuffer;	


	// Ain't it nice to have friends...
	friend class SpriteGroup;
};



/**
 * SpriteGroup assists in the management of a collection of sprites.
 */
class SpriteGroup
{
	public:
	
	
	/**
	 * Create a SpriteGroup. 
	 *
	 * @param numSprites the number of managed sprites
	 */
	SpriteGroup(UInt16 numSprites) SEC_RAZOR_INIT;
	
	
	/**
	 * Destroy the SpriteGroup and deallocate all used resources.
	 */
	~SpriteGroup() SEC_RAZOR_INIT;


	/**
	 * Create an animated sprite with the specified AnimFrames. Ownership of the AnimFrames object is <em>NOT</em>
	 * transferred to the Sprite. It will not automatically be destroyed when the Sprite is destroyed.
	 *
	 * @param animFrames a reference to an AnimFrames object
	 * @param visible shall the sprite be visible?
     * @param saveBackground shall the sprite's background be preserved during a draw operation?
	 */
	void createSprite(UInt16 spriteIdx, const AnimFrames *animFrames, Boolean visible = true, Boolean saveBackground = false) SEC_RAZOR_INIT;

	/**
	 * Create a group of identical animated sprites with the specified AnimFrames. 
	 * All Sprites in the SpriteGroup will be set to the specified properties.
	 * Ownership of the AnimFrames object is <em>NOT</em>
	 * transferred to the Sprite. It will not automatically be destroyed when the Sprite is destroyed.
	 *
	 * @param animFrames a reference to an AnimFrames object
	 * @param visible shall the sprite be visible?
     * @param saveBackground shall the sprite's background be preserved during a draw operation?
	 */
	void createSprites(const AnimFrames *animFrames, Boolean visible = true, Boolean saveBackground = false) SEC_RAZOR_INIT;

/** @name State management (visibility, position, etc.)
 */
//@{
	/**
	 * Show the sprite during subsequent draws.
	 */
	void show(UInt16 spriteIdx);


	/**
	 * Hide the sprite during subsequent draws.
	 */
	void hide(UInt16 spriteIdx);


	/**
	 * Set the visibility of the sprite through a flag.
     *
	 * @param visible shall the sprite be visible (true = shown / false = hidden)
	 */
	Boolean setVisibility(UInt16 spriteIdx, Boolean visible);


	/**
	 * Will the sprite be drawn during subsequent draws?
	 */
	Boolean isVisible(UInt16 spriteIdx) const;


	/**
	 * Set the displayed frame of the Sprite.
	 *
	 * @param frameIndex the index of the displayed frame. Must be between 0..numFrames-1 of the associated AnimFrames.
	 */
	void setFrame(UInt16 spriteIdx, UInt16 frameIndex);


	/**
	 * Move the sprite (i.e. its hotspot) to the specified coordinates.
	 */
	void move(UInt16 spriteIdx, Coord x, Coord y);
//@}


	/**
	 * Draw all the sprites in the group.
	 */
	void draw(RectangleType *bounds = NULL) SEC_RAZOR;

	/**
	 * Restore the background of all sprites in the group.
	 */
	void restoreBackground() const SEC_RAZOR;


	private:

	UInt16 numSprites;
	Sprite *sprites;
};



#endif
